#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun 21 12:17:35 2023

@author: radomirpestow
"""


import numpy as np
from scipy.integrate import solve_ivp
from numba import njit

from copy import copy

# This is the right hand sinde of the ODE system
@njit(cache = True)
def f(t, variables, parameters):
    S, I, R, W = variables 
    init_infected, beta, gamma, theta, bias = parameters
    
    action = min(bias*theta*beta*I/2,1)
    
    dS = -beta*(1-action)*I*S
    dI = beta*(1-action)*I*S - gamma*I #beta(1-action)*S/gamma >=< 1 diesen term plotten
    dR = gamma*I
    dW = (-action**2 - theta*beta*(1-action)*I)*S
    
    return dS, dI, dR, dW

def is_valid(parameters):
    p = parameters
    return p["bias"]*p["severity"]*p["beta"]*p["init_infected"]/2 < 1

def vary_parameter(parameters, name, parameter_list):
    """varies the parameter with the given name
    
    Parameters
    ----------
    parameters : parameter dictionary
    name : name of the parameter to vary (string)
    space : iterable with parameters values to assume

    Returns
    -------
    lst : list parameters

    """
    lst = []
    for p in parameter_list: 
        tmp = copy(parameters)
        tmp[name] = p
        assert is_valid(tmp)
        lst.append(tmp)
    return lst

def evaluate(params, time):
    """Evaluates the parameter list and appends the result
    
    Parameters
    ----------
    parameter_list : list of parameter dictionaries
    """
    iv = (1 - params["init_infected"], 
              params["init_infected"],
              0, # recovered
              0) # initial welfare
              
    tuple(params.values())
    sol = solve_ivp(f, (0, time), iv, args = (tuple(params.values()),), max_step = 1) #, max_step=0.1
    params["t"] = sol.t 
    params["y"] = sol.y

    params["finished"] = (True if params["y"][1, -1] < params["init_infected"] else False)
    return params

def is_finished(model):
    """
    Returns true if the simulation is finished
    """
    return model["finished"]

def evaluate_list(param_list, time):
    """Evaluates the parameter list and appends the result
    
    Parameters
    ----------
    parameter_list : list of parameter dictionaries
    """
    for params in param_list:
        evaluate(params, time)

        # while not is_finished(params) and time < max_time:
        #     time = time*2
        #     del(params["t"])
        #     del(params["y"])
        #     evaluate(params, time)
        






